@@ -13,6 +13,7 @@ from azure.storage.blob import BlobClient
 from azure.cosmosdb.table import TableService, TableBatch
 from typing import List
 from multiprocessing.pool import ThreadPool
+from azure.core.exceptions import ResourceExistsError
 
 enough_workers_message = "There are already enough active workers"
 workers_long_running = "Workers are active for too long. Going to process as well."
@@ -77,31 +78,39 @@ def singleton(class_):
 
 
 class Worker:
-    def __init__(self, queue_name, metadata_table_name, worker_name, segment_limit=200):
+    def __init__(self, worker_name, segment_limit=200):
         self.connection_string = os.environ['AzureWebJobsStorage']
-        self.metadata_table_name = metadata_table_name
-        self.table_client = TableService(connection_string=self.connection_string)
-        self.queue = QueueClient.from_connection_string(conn_str=self.connection_string, queue_name=queue_name)
+        self.metadata_table_name = worker_name + "Data"
+        self.queue_name = worker_name.lower()
         self.worker_name = worker_name
         self.segment_limit = segment_limit
+
+        self.table_client = TableService(connection_string=self.connection_string)
+        self.table_client.create_table(self.metadata_table_name)
+        try:
+            self.queue = QueueClient.from_connection_string(conn_str=self.connection_string, queue_name=self.queue_name)
+            self.queue.create_queue()
+        except ResourceExistsError:
+            pass
+
         self.pool = ThreadPool(processes=1)
 
-    def trigger_blob(self, blob_name, data=None):
-        request_id = self._insert_blob_metadata(blob_name, data)
+    def trigger_blob(self, blob_name, params=None):
+        request_id = self._insert_blob_metadata(blob_name, params)
         self._enqueue(request_id)
         return request_id
     
-    def trigger_blobs(self, blob_names: List[str], data=None):
+    def trigger_blobs(self, blob_names: List[str], params=None):
         guid = self._generate_id()
         request_ids = []
         for blob_name in blob_names:
-            request_id = self._insert_blob_metadata(blob_name, data, guid)
+            request_id = self._insert_blob_metadata(blob_name, params, guid)
             self._enqueue(request_id)
             request_ids.append(request_id)
         return request_ids
 
-    def trigger(self, data):
-        request_id = self._insert_request_metadata(data)
+    def trigger(self, data, params):
+        request_id = self._insert_request_metadata({"Data": data, "Params": params})
         self._enqueue(request_id)
         return request_id
         
@@ -114,7 +123,7 @@ class Worker:
         entities = list(self.table_client.query_entities(self.metadata_table_name, filter="PartitionKey eq '%s'" % guid))
         return [TaskResult.create_from_request_entity(entity) for entity in entities]
 
-    def process_messages_stateful(self, func, blob_func=None):
+    def process_messages_stateful(self, func):
         state_row = list(self.table_client.query_entities(state_table))[0]
         now = datetime.datetime.now(datetime.timezone.utc)
         active_count = state_row['ActiveCount'].value
@@ -140,21 +149,21 @@ class Worker:
                     break
                 for message in messages:
                     content = base64.b64decode(message.content).decode("utf-8")
-                    self.process_message(content, func, blob_func)
+                    self.process_message(content, func)
                     self.queue.delete_message(message)
         finally:
             state_row['ActiveCount'].value = 0
             state_row['UpdateDate'] = datetime.datetime.now(datetime.timezone.utc)
             self.table_client.update_entity(state_table, state_row)
 
-    def process_message(self, request_id, func, blob_func=None):
+    def process_message(self, request_id, func):
         try:
             guid, request_hash = self._decode_id(request_id)
             request_entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash)
             if 'Request' in request_entity:
                 self._process_request_body(request_entity, func)
             if 'BlobName' in request_entity:
-                self._process_blob(request_id, request_entity, blob_func)
+                self._process_blob(request_id, request_entity, func)
         except Exception as e:
             self._record_request_failure(request_entity, e)
             self._record_failure(request_id, e)
@@ -213,8 +222,8 @@ class Worker:
     def _read_metadata(self, request_entity):
         blob_name = request_entity['BlobName']
         params, offset, retry_count, batch_count, next_chunk = None, 0, 0, 0, None
-        if 'Parameters' in request_entity:
-            params = json.loads(request_entity['Parameters'])
+        if 'Params' in request_entity:
+            params = json.loads(request_entity['Params'])
         if 'Offset' in request_entity:
             offset = request_entity['Offset']
         if 'RetryCount' in request_entity:
@@ -302,11 +311,11 @@ class Worker:
         self.table_client.insert_entity(self.metadata_table_name, metadata)
         return request_id
 
-    def _insert_blob_metadata(self, blob_name, data=None, guid=None):
+    def _insert_blob_metadata(self, blob_name, params=None, guid=None):
         guid, request_hash, request_id = self._generate_request_id(blob_name, guid)
         metadata_entity = {"PartitionKey": guid, "RowKey": request_hash, "BlobName": blob_name}
-        if data:
-            metadata_entity['Parameters'] = json.dumps(data)
+        if params:
+            metadata_entity['Params'] = json.dumps(params)
         self.table_client.insert_entity(self.metadata_table_name, metadata_entity)
         return request_id
 
@@ -314,10 +323,10 @@ class Worker:
         self.queue.send_message(base64.b64encode(request_id.encode()).decode())
 
     def _process_request_body(self, request_entity, func):
-        if 'Request' not in request_entity:
+        if "Request" not in request_entity:
             return
-        request_body = json.loads(request_entity['Request'])
-        response = func(request_body)
+        request_body = json.loads(request_entity["Request"])
+        response = func(request_body["Data"], request_body["Params"])
         request_entity['Result'] = json.dumps(response)
         self.table_client.update_entity(self.metadata_table_name, request_entity)
 
